"""Add single column id for domain objects such as actions, slots, etc.

Reason:
Currently the primary keys for these tables are composite keys of the associated domain
and the name of the object (e.g. the action name). If we don't want to use the domain
id in our API, we need single column primary keys to identify an object.

Revision ID: 8b528d137850
Revises: b4a9bd47b6e0

"""
import warnings

from typing import Text
from alembic import op
from alembic.operations import Operations
import sqlalchemy as sa
from sqlalchemy.dialects.sqlite.pysqlite import SQLiteDialect

# revision identifiers, used by Alembic.
revision = "8b528d137850"
down_revision = "b4a9bd47b6e0"
branch_labels = None
depends_on = None


def upgrade():
    # Filter warnings for inconsistent state of primary keys
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", category=sa.exc.SAWarning)
        with op.batch_alter_table("domain_action") as batch_op:
            batch_op.alter_column("id", new_column_name="domain_id")
            batch_op.add_column(sa.Column("action_id", sa.Integer, primary_key=True))
            _drop_primary_key("domain_action_pkey", batch_op)
            batch_op.create_primary_key("domain_action_pk", columns=["action_id"])

        with op.batch_alter_table("domain_intent") as batch_op:
            batch_op.alter_column("id", new_column_name="domain_id")
            batch_op.add_column(sa.Column("intent_id", sa.Integer, primary_key=True))
            _drop_primary_key("domain_intent_pkey", batch_op)
            batch_op.create_primary_key("domain_intent_pk", columns=["intent_id"])

        with op.batch_alter_table("domain_entity") as batch_op:
            batch_op.alter_column("id", new_column_name="domain_id")
            batch_op.add_column(sa.Column("entity_id", sa.Integer, primary_key=True))
            _drop_primary_key("domain_entity_pkey", batch_op)
            batch_op.create_primary_key("domain_entity_pk", columns=["entity_id"])

        with op.batch_alter_table("domain_slot") as batch_op:
            batch_op.alter_column("id", new_column_name="domain_id")
            batch_op.add_column(sa.Column("slot_id", sa.Integer, primary_key=True))
            _drop_primary_key("domain_slot_pkey", batch_op)
            batch_op.create_primary_key("domain_slot_pk", columns=["slot_id"])


def downgrade():
    with op.batch_alter_table("domain_action") as batch_op:
        _drop_primary_key("domain_action_pk", batch_op)
        batch_op.create_primary_key("domain_action_pk", columns=["domain_id", "action"])
        batch_op.drop_column("action_id")
        batch_op.alter_column("domain_id", new_column_name="id")

    with op.batch_alter_table("domain_intent") as batch_op:
        _drop_primary_key("domain_intent_pk", batch_op)
        batch_op.create_primary_key("domain_intent_pk", columns=["domain_id", "intent"])
        batch_op.drop_column("intent_id")
        batch_op.alter_column("domain_id", new_column_name="id")

    with op.batch_alter_table("domain_entity") as batch_op:
        _drop_primary_key("domain_entity_pk", batch_op)
        batch_op.create_primary_key("domain_entity_pk", columns=["domain_id", "entity"])
        batch_op.drop_column("entity_id")
        batch_op.alter_column("domain_id", new_column_name="id")

    with op.batch_alter_table("domain_slot") as batch_op:
        _drop_primary_key("domain_slot_pk", batch_op)
        batch_op.create_primary_key("domain_slot_pk", columns=["domain_id", "slot"])
        batch_op.drop_column("slot_id")
        batch_op.alter_column("domain_id", new_column_name="id")


def _drop_primary_key(key_name: Text, batch_op: Operations) -> None:
    context = op.get_context()
    # in SQLite the constraint is unnamed and this does not work
    # (dropping the constraint is also not needed)
    if not isinstance(context.dialect, SQLiteDialect):
        batch_op.drop_constraint(key_name)
